use aes_gcm::{aead::Aead, Aes256Gcm, Key, KeyInit, Nonce};

use super::SgxSealing;
use crate::SgxError;

impl SgxSealing {
    pub fn un_seal_data(self, payload_encrypted_packet: Vec<u8>) -> Result<Vec<u8>, SgxError> {
        let sealing_key = self.get_aes256_sealing_key()?;

        let aes_encryption_256bit_key = Key::<Aes256Gcm>::from_slice(&sealing_key);
        let nonce = Nonce::from_slice(&payload_encrypted_packet[..12]); // nonce (IV) 96-bits (12); is prepending while encryping
        let raw_cipher_text = &payload_encrypted_packet[12..];

        let cipher = Aes256Gcm::new(aes_encryption_256bit_key);
        let clear_text = cipher
            .decrypt(nonce, raw_cipher_text.as_ref())
            .map_err(|e| SgxError::UnSealingError(e.to_string()))?;

        Ok(clear_text)
    }
}
