use aes_gcm::{
    aead::{Aead, AeadCore, OsRng},
    Aes256Gcm, Key, KeyInit,
};

use super::SgxSealing;
use crate::SgxError;

impl SgxSealing {
    pub fn seal_data(self, payload_plain_text: Vec<u8>) -> Result<Vec<u8>, SgxError> {
        let sealing_key = self.get_aes256_sealing_key()?;

        let aes_encryption_256bit_key = Key::<Aes256Gcm>::from_slice(&sealing_key);
        let nonce = Aes256Gcm::generate_nonce(&mut OsRng); // 96-bits (12); unique per message

        let cipher = Aes256Gcm::new(aes_encryption_256bit_key);
        let mut ciphertext = cipher
            .encrypt(&nonce, payload_plain_text.as_ref())
            .map_err(|e| SgxError::SealingError(e.to_string()))?;

        let mut encrypted_packet = nonce.to_vec();
        encrypted_packet.append(&mut ciphertext);

        Ok(encrypted_packet)
    }
}
